/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#ifndef DATA_H_
#define DATA_H_
#include <map>
#include <vector>
#include <string>
#include <list>
#include "Options.h"
#include "DoubleMatrix.h"
#include "DoubleVector.h"
#include "FeatureSet.h"

namespace models {
	class Model;
}

namespace data {

class Observation {
public:
	
	std::vector<std::string> cp;
	std::vector<int> features;
	//std::vector<short> cp;
	//std::map<int,double> features;
	//boost::dynamic_bitset<> feat_mask;
	
	int label;
	int prediction;
	
	Observation(){
		//feat_mask.resize(2);
	}
	virtual ~Observation(){
	}
	
	void compute_state_feature_scores(features::FeatureSet * fs, std::vector<Observation> & seq, int pos, mathlib::DoubleVector * Vi, double * weights);
	void compute_transition_feature_scores(features::FeatureSet * fs, mathlib::DoubleMatrix * Mi, double * weights);
};

typedef std::vector<Observation> Sequence;
typedef std::list<Sequence> Dataset;

class Data
{
public:
	
	models::Model * model;
	
	int num_sequences;
	Dataset * ds;
	Data();
	~Data();
	void read(std::string sequences_file);
	void dump();
	void write_predictions();
};

}

#endif /*DATA_H_*/

