/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#ifndef DOUBLEMATRIX_H_
#define DOUBLEMATRIX_H_

#include "DoubleVector.h"
#include <iostream>


namespace mathlib
{
class DoubleMatrix
{
public:
	//gsl_matrix * mtrx;
	double **mtrx;
    int rows, cols;	// number of rows and columns
    
    // default constructor
    DoubleMatrix() {
	rows = cols = 0;
	mtrx = NULL;
    }

    // constructor with rows and cols    
    DoubleMatrix(int rows, int cols);
    
    // destructor
    ~DoubleMatrix(){
    	if (mtrx != NULL) {
    	 for (int i=0; i<rows; i++){
    		if (mtrx[i] != NULL) {
    		delete [] mtrx[i];
    		}
    	 }
    	delete [] mtrx;
    	}
    }
    
    // overloading assignment operator (with one value)
    void operator=(double val);
    
    void operator=(DoubleMatrix & m);
    
    // getting an element
    double get(int i, int j);
    
    // setting an element
    void set(int i, int j, double value);
	
	// setting a whole column to value
	void set_column(int column, double value);
	
	// setting a column to v
	void set_column(int column, DoubleVector * v);
	
	// 
	void set_row(int row, DoubleVector * v);

	// 
	void set_row(int row, double value);
	
	void multiply_entrywise(DoubleMatrix * m, int direction);
	
	// multiply a DoubleMatrix with a DoubleVector
	void multiply(mathlib::DoubleVector * result, DoubleVector * v, 
					int transposition);
	
	void multiply(mathlib::DoubleMatrix * result, int column, DoubleVector * v, 
					int transposition);
	
	//
	void column_multiply(int column, DoubleVector * v, int direction);
	
	//
	void column_multiply(int column, double value);
	
	//
	void row_multiply(int row, DoubleVector * v, int direction);
	
	double sum_row(int row);
	double sum_column(int column);
	void sum_column(int column, DoubleVector * vect);
	
	void get_row(mathlib::DoubleVector * row_vector, int row);
	void get_column(mathlib::DoubleVector * column_vector, int column);
	
	void dump();
	void dump_column(int j);
};

}

#endif /*DOUBLEMATRIX_H_*/

