/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * FeatureSet.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <iostream>
#include <stdlib.h>
#include "Model.h"
#include "FeatureSet.h"
#include "Data.h"
#include "Tokenizer.h"
#include "Conversions.h"



namespace features {
FeatureSet::FeatureSet()
{
}

FeatureSet::FeatureSet(std::ifstream * model_file)
{
	char buffer[BUFF_SIZE_LONG];
	
	model_file->getline(buffer, BUFF_SIZE_LONG);
	read_dictionary(model_file);
	model_file->getline(buffer, BUFF_SIZE_LONG);
	read_features(model_file);
}

FeatureSet::~FeatureSet()
{
}

void FeatureSet::read_features(std::ifstream * model_file)
{
	utils::Tokenizer * tok;
	std::string line;
	char buffer[BUFF_SIZE_LONG];
	
	model_file->getline(buffer, BUFF_SIZE_LONG);
	line = buffer;
	tok = new utils::Tokenizer(line, ":\n");
	
	while (tok->token(0).compare("#END_FEATURESET") != 0) {
		int key, type, from, to, cp, window;
		double weight;
		type = atoi(tok->token(0).c_str());
		key = atoi(tok->token(1).c_str());
		from = atoi(tok->token(2).c_str());
		to = atoi(tok->token(3).c_str());
		cp = atoi(tok->token(4).c_str());
		window = atoi(tok->token(5).c_str());
		weight = atof(tok->token(6).c_str());
		if (type == STATE_FEATURE) {
				Feature f(to, window, cp);
				f.weight = weight;
				f.key = key;
				this->features.push_back(f);
				
				std::string strid = "";	
				strid += utils::Conversions::int2str(window)+"#";
				strid += utils::Conversions::int2str(cp)+"#";
				strid += utils::Conversions::int2str(to)+"#";
				
				this->fid_map.insert(std::pair<std::string,int>(strid,key));
		} else {	
				
				Feature f(to, from);
				f.weight = weight;
				f.key = key;
				this->features.push_back(f);
				//this->trans_features.push_back(f);
				this->trans_features.push_back(f.key);
				std::string strid("");	
				strid += utils::Conversions::int2str(from)+"#";
				strid += utils::Conversions::int2str(to)+"#";
				
				this->fid_map.insert(std::pair<std::string,int>(strid,key));
				//this->tfv.insert(std::pair<int,double>(key,1.0));
		}
		
		model_file->getline(buffer, BUFF_SIZE_LONG);
		line = buffer;
		delete tok;
		tok = new utils::Tokenizer(line, ":\n");
	}
	
	delete tok;
	
	this->num_features = features.size();
}

void FeatureSet::read_dictionary(std::ifstream * model_file)
{
	utils::Tokenizer * tok;
	std::string line;
	char buffer[BUFF_SIZE_LONG];
	
	model_file->getline(buffer, BUFF_SIZE_LONG);
	line = buffer;
	tok = new utils::Tokenizer(line, ":\n");
	
//	this->model->options->window_size = atoi(tok->token(1).c_str()); 
	
	while (tok->token(0).compare("#END_DICTIONARY") != 0) {
		
		int cp = atoi(tok->token(0).c_str());
		int window_offset = atoi(tok->token(1).c_str());
		int to_state = atoi(tok->token(2).c_str());
		this->dictionary.insert(std::pair<int, std::pair<int, int> >(cp, std::pair<int, int>(window_offset, to_state)));
		
		
		model_file->getline(buffer, BUFF_SIZE_LONG);
		line = buffer;
		delete tok;
		tok = new utils::Tokenizer(line, ":\n");
	}
	
	delete tok;
}

void FeatureSet::gen_features()
{
	int seq, pos, i, j, k;
	data::Dataset::iterator datait;
	
	grammar::Transducer * tr = this->model->transducer;
	options::Options * opt = this->model->options;
	data::Data * dt = this->model->data;
	
	seq = pos = 0;
	
	int k1 = add(0, -1, 0);
	int k2 = add(0, -2, 1);
	
	// Generate transition features 
	for (i = 0; i < tr->num_states; i++) {
		for (j = 0; j < tr->num_states; j++) {
			if (tr->transitions->get(i, j) == 1.0) {
				int key = add(i, j);
				//tfv.insert(std::pair<int,double>(key,1.0));
			}
		}
	}
	
	
	// Generate state features
	for (datait = dt->ds->begin() ; 
			datait != dt->ds->end(); datait++) {
	
		int seqlen = datait->size();
		
		
		//std::cout << "Simulate sequence " << seq << std::endl;
		tr->simulate(*datait);

		//(*datait)[0].feat_mask.resize(2);
		//(*datait)[seqlen-1].feat_mask.resize(2);
		
		//(*datait)[0].feat_mask[(boost::dynamic_bitset<>::size_type) k1] = 1;
		//(*datait)[seqlen-1].feat_mask[(boost::dynamic_bitset<>::size_type) k2] = 1;
		
		//(*datait)[0].features.insert(std::pair<int, double>(k1, 1.0));
		//(*datait)[seqlen-1].features.insert(std::pair<int, double>(k2, 1.0));
		(*datait)[seqlen-1].features = trans_features;

		(*datait)[0].features.push_back(k1);
		(*datait)[seqlen-1].features.push_back(k2);
		
		for (pos = 1 ; pos < seqlen-1; pos++) {
				
			int windown = pos - opt->window_size;
			if (windown < 1) windown = 1;
			int winup = pos + opt->window_size;
			if (winup > seqlen-2) winup = seqlen-2;
			
			std::vector<int> states = tr->get_states(pos);
			
			// Ranging over whole window ...
			for (i = windown; i <= winup ; i++) {
				
				// For each position in the feature vectors ...
				for (j = 0; j < (*datait)[i].cp.size() ; j++) {	
					
					// Generate a state feature for each state may occur at position pos
					for (k = 0; k < states.size(); k++) {
						add(i-pos, j, states[k]);
					}
				}
			}
		}
		seq++;
	}
	
	this->num_features = features.size();
}
/*
void FeatureSet::assign_features_multi_threaded()
{
	int i;
	boost::shared_ptr<boost::thread> * threads = new boost::shared_ptr<boost::thread>[model->options->cpu_cores];
	
	for (i=0; i<model->options->cpu_cores; i++)
		threads[i] = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&FeatureSet::do_thread, this, i)));
	
	for (i=0; i<model->options->cpu_cores; i++)
		threads[i]->join();
	
}
*/
/*
void FeatureSet::do_thread(int index)
{
	int pos, j;
	data::Dataset::iterator datait;
	std::map<std::string,int>::iterator fidmit;
	
	std::pair<std::multimap<int,std::pair<int, int> >::iterator,std::multimap<int,std::pair<int, int> >::iterator> ret;
    std::multimap<int,std::pair<int, int> >::iterator dictit;
	
	options::Options * opt = this->model->options;
	data::Data * dt = this->model->data;
	
	int nseqs = model->data->ds->size();
	
	int seqs_per_thread = nseqs / model->options->cpu_cores;
	int rest = nseqs % model->options->cpu_cores;
	
	int start = index*seqs_per_thread;
	int end = (index+1)*seqs_per_thread;
	if (index == model->options->cpu_cores-1)
		end += rest;
	int seq = 0;
	
	std::cout << "Thread " << index << " starting assigning features on sequences range " << start << "-" << end <<std::endl;
	// Generate state features
	for (datait = dt->ds->begin() ; 
			datait != dt->ds->end() && seq < end; datait++) {
		
		if (seq < start) {
			seq++;
			continue;	
		}
				
		int seqlen = datait->size();

		std::cout << "Thread " << index << " sequence " << seq  << " len " << seqlen << std::endl;
		(*datait)[0].feat_mask.resize((boost::dynamic_bitset<>::size_type) ((*datait)[0].feat_mask.size()+trans_features.size()));
		(*datait)[seqlen-1].feat_mask.resize((boost::dynamic_bitset<>::size_type) ((*datait)[seqlen-1].feat_mask.size()+trans_features.size()), true);
		//(*datait)[seqlen-1].features = tfv;
		//std::cout << "Assigning feature to sequence" << std::endl;
		//int del = 1;
		for (pos = 1 ; pos < seqlen-1; pos++) {
				
			int windown = pos - opt->window_size;
			if (windown < 1) windown = 1;
			int winup = pos + opt->window_size;
			if (winup > seqlen-2) winup = seqlen-2;
			//(*datait)[pos].features = tfv;
			//(*datait)[pos].feat_mask.resize(2);
			(*datait)[pos].feat_mask.resize((boost::dynamic_bitset<>::size_type) ((*datait)[pos].feat_mask.size()+trans_features.size()), true);
			for (j = 0; j < (*datait)[pos].cp.size() ; j++) {	
				ret = dictionary.equal_range(j);
				for (dictit = ret.first; dictit != ret.second; ++dictit) {
					if (dictit->second.first+pos >= windown && dictit->second.first+pos <= winup){
					std::string strid = "";
					strid += utils::Conversions::int2str(dictit->second.first)+"#";
					strid += utils::Conversions::int2str(j)+"#";
					strid += utils::Conversions::int2str(dictit->second.second)+"#";
					
					fidmit = fid_map.find(strid);
					if (fidmit != fid_map.end()) {
						//double val = atof((*datait)[dictit->second.first+pos].cp[j].c_str());
						double val = (double) (*datait)[dictit->second.first+pos].cp[j] / 100.0;
						if (val > 0.0){
						//(*datait)[pos].features.insert(std::pair<int, double>(fidmit->second, val));
						if (fidmit->second+1 > (*datait)[pos].feat_mask.size()){
						(*datait)[pos].feat_mask.resize((boost::dynamic_bitset<>::size_type) fidmit->second+1);
						}
						(*datait)[pos].feat_mask[(boost::dynamic_bitset<>::size_type) fidmit->second] = 1;
						}
					}
					}
				}
			}
		}
		seq++; 
	}
}*/

void FeatureSet::assign_features() 
{
	int pos, j;
	data::Dataset::iterator datait;
	std::map<std::string,int>::iterator fidmit;
	
	std::pair<std::multimap<int,std::pair<int, int> >::iterator,std::multimap<int,std::pair<int, int> >::iterator> ret;
    	std::multimap<int,std::pair<int, int> >::iterator dictit;
	
	options::Options * opt = this->model->options;
	data::Data * dt = this->model->data;
	
	
	// Generate state features
	for (datait = dt->ds->begin() ; 
			datait != dt->ds->end(); datait++) {
		int seqlen = datait->size();
		//(*datait)[0].feat_mask.resize((boost::dynamic_bitset<>::size_type) ((*datait)[0].feat_mask.size()+trans_features.size()));
		//(*datait)[seqlen-1].feat_mask.resize((boost::dynamic_bitset<>::size_type) ((*datait)[seqlen-1].feat_mask.size()+trans_features.size()), true);
		//(*datait)[seqlen-1].features = tfv;
		//std::cout << "Assigning feature to sequence" << std::endl;
		//int del = 1;
		(*datait)[seqlen-1].features = trans_features;
		
		for (pos = 1 ; pos < seqlen-1; pos++) {
				
			int windown = pos - opt->window_size;
			if (windown < 1) windown = 1;
			int winup = pos + opt->window_size;
			if (winup > seqlen-2) winup = seqlen-2;
			//(*datait)[pos].features = tfv;
			//(*datait)[pos].feat_mask.resize(2);
			//(*datait)[pos].feat_mask.resize((boost::dynamic_bitset<>::size_type) ((*datait)[pos].feat_mask.size()+trans_features.size()), true);
			(*datait)[pos].features = trans_features;
			for (j = 0; j < (*datait)[pos].cp.size() ; j++) {	
				ret = dictionary.equal_range(j);
				for (dictit = ret.first; dictit != ret.second; ++dictit) {
					if (dictit->second.first+pos >= windown && dictit->second.first+pos <= winup){
					std::string strid = "";
					strid += utils::Conversions::int2str(dictit->second.first)+"#";
					strid += utils::Conversions::int2str(j)+"#";
					strid += utils::Conversions::int2str(dictit->second.second)+"#";
					
					fidmit = fid_map.find(strid);
					if (fidmit != fid_map.end()) {
						double val = atof((*datait)[dictit->second.first+pos].cp[j].c_str());
						//double val = (double) (*datait)[dictit->second.first+pos].cp[j] / 100.0;
						if (val > 0.0){
						//(*datait)[pos].features.insert(std::pair<int, double>(fidmit->second, val));
						/*
						if (fidmit->second+1 > (*datait)[pos].feat_mask.size()){
						(*datait)[pos].feat_mask.resize((boost::dynamic_bitset<>::size_type) fidmit->second+1);
						}
						(*datait)[pos].feat_mask[(boost::dynamic_bitset<>::size_type) fidmit->second] = 1;
						*/
						(*datait)[pos].features.push_back(fidmit->second);
						}
					}
					}
				}
			}
			/*
			if (pos - del == opt->window_size){
			  (*datait)[del].cp.clear();
			  del++;
			}*/
		}
		/*
		for ( ; del<seqlen-1; del++)
			(*datait)[del].cp.clear();
			*/
	}
}

int FeatureSet::add(int window_offset, int cp, int to_state)
{
	int res;
	
	std::string strid = "";
	std::map<std::string,int>::iterator fidmit;
	
	
	strid += utils::Conversions::int2str(window_offset)+"#";
	strid += utils::Conversions::int2str(cp)+"#";
	strid += utils::Conversions::int2str(to_state)+"#";
	
	fidmit = fid_map.find(strid);
	if (fidmit == fid_map.end()) {
		Feature f(to_state, window_offset, cp);
		f.key = fid_map.size();
		fid_map.insert(std::pair<std::string,int>(strid,f.key));
		features.push_back(f);
		res = f.key;
		dictionary.insert(std::pair<int, std::pair<int, int> >(cp, std::pair<int, int>(window_offset, to_state)));
	} else {
		res = fidmit->second;
	}
	return res;
}

int FeatureSet::add(int from_state, int to_state)
{
	int res;
	
	std::string strid("");
	std::map<std::string,int>::iterator fidmit;
	
	strid += utils::Conversions::int2str(from_state)+"#";
	strid += utils::Conversions::int2str(to_state)+"#";
	
	fidmit = fid_map.find(strid);
	if (fidmit == fid_map.end()) {
		Feature f(to_state, from_state);
		f.key = fid_map.size();
		fid_map.insert(std::pair<std::string,int>(strid,f.key));
		features.push_back(f);
		//trans_features.push_back(f);
		trans_features.push_back(f.key);
		res = f.key;
	} else {
		res = fidmit->second;
	}
	
	return res;
}

void FeatureSet::dump() 
{
	int i;
	
	
	
	for (i = 0; i < features.size(); i++) {
		std::cout << "Feature {" << std::endl;
		std::cout << "key    : " << features[i].key << std::endl;
		std::cout << "from   : " << features[i].from_state << std::endl;
		std::cout << "to     : " << features[i].to_state << std::endl;
		std::cout << "cp     : " << features[i].cp << std::endl;
		std::cout << "win    : " << features[i].window_offset << std::endl;
		std::cout << "type   : " << features[i].type << std::endl;
		std::cout << "weight : " << features[i].weight << std::endl;
		std::cout << "}" << std::endl;
	}
	
	std::map<std::string,int>::iterator fmit;
	for (fmit = fid_map.begin(); fmit != fid_map.end(); fmit++)
	{
		std::cout << fmit->first << " " << fmit->second << std::endl;
	}
}

void FeatureSet::dump(std::ofstream * os)
{
	int i;
	std::multimap<int,std::pair<int, int> >::iterator dictit;
	
	*os << "#WINDOW " << this->model->options->window_size << std::endl;
	*os << "#DICTIONARY" << std::endl;
	
	for (dictit = this->dictionary.begin(); 
				dictit != this->dictionary.end(); dictit++) {
		*os << dictit->first << ":" << dictit->second.first << ":" << dictit->second.second << std::endl;
	}
	*os << "#END_DICTIONARY" << std::endl;
	
	*os << "#FEATURESET" << std::endl;
	for (i = 0; i < features.size(); i++) {
		*os << features[i].type << ":";
		*os << features[i].key << ":";
		*os << features[i].from_state << ":";
		*os << features[i].to_state << ":";
		*os << features[i].cp << ":";
		*os << features[i].window_offset << ":";
		//*os << features[i].weight;
		char w[64];
		sprintf(w, "%.5f ", features[i].weight);
		*os << w;
		*os << std::endl;
	}
	*os << "#END_FEATURESET" << std::endl;
}
}
