/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * Options.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include "Tokenizer.h"
#include "Options.h"

namespace options {

Options::Options()
{
	training_file = "training.set";
        testing_file = "testing.set";
        unlabelled_file = "unlabelled.set";
        topology_file = "topology.model";

        model_file = "model.txt";

        istraining = false;
        istesting = false;
        ispredicting = false;
	out_posterior = false;
	
	window_size = 0;
	sigma_square = 0.5;
	init_params_value = 0.0;
	epsilon = 0.0001;
	iterations = 60;

	model = GRCRFM;

        training_algorithm = DG;
	decoding_algorithm = VITERBI;
        include_bos_feature = true;
        include_eos_feature = true;

        randomize_params = false;
        regularize = true;

        mhessian = 7;

        is_scaling = true;

        optimizer = LBFGS;
}

Options::Options(std::string options_file)
{	
	training_file = "training.set";
	testing_file = "testing.set";
	unlabelled_file = "unlabelled.set";
	topology_file = "topology.model";
	
	model_file = "model.txt";
	train_logfile = "training.log";
	decode_logfile = "decoding.log";
	cv_file = "cv.txt";
	test_session_file = "test_session.txt";
	
	istraining = false;
	istesting = false;
	ispredicting = false;
	isretraining = false;
	out_posterior = false;	
	model = GRCRFM;
	
	window_size = 0;
	
	training_algorithm = DG;
	
	decoding_algorithm = VITERBI;
	include_bos_feature = true;
	include_eos_feature = true;
	
	iterations = 60;
	init_params_value = 0.0;
	randomize_params = false;
	regularize = true;
	sigma_square = 0.5;
	
	epsilon = 0.0001;
	mhessian = 7;
	
	is_scaling = true;
	
	optimizer = LBFGS;
	
	read(options_file);
}

Options::~Options()
{
}

void Options::read(std::string options_file)
{
	std::string line;
	char buffer[BUFF_SIZE_LONG];
	std::filebuf fb;
	fb.open(options_file.c_str(), std::ios::in);
	std::istream is(&fb);
	
	while (! is.getline(buffer, BUFF_SIZE_LONG).eof()) {
		line = buffer;
		
		// find '#' character: if found, the current line is a comment
		int i = 0;
		while (i < line.size() && (line[i] == ' ' || line[i] == '\t')) {
		    i++;
		}	
		if (i < line.size() && line[i] == '#') {
		    continue;
		}
		
		utils::Tokenizer tok(line, "= \t\r\n");
		int len = tok.count_tokens();
		if (len != 2) {
		    // invalid, ignore this line
		    continue;
		}
		
		std::string optstr = tok.token(0);
		std::string optval = tok.token(1);
		
		if (optstr.compare("training_file") == 0) {
			this->training_file = optval;
		} else if (optstr.compare("testing_file") == 0) {
			this->testing_file = optval;
		} else if (optstr.compare("unlabelled_file") == 0) {
			this->unlabelled_file = optval;
		} else if (optstr.compare("topology_file") == 0) {
			this->topology_file = optval;
		} else if (optstr.compare("model_file") == 0) {
			this->model_file = optval;
		} else if (optstr.compare("prediction_file") == 0) {
			this->prediction_file = optval;
		} else if (optstr.compare("model") == 0) {
			this->model = atoi(optval.c_str());
		} else if (optstr.compare("window_size") == 0) {
			this->window_size = atoi(optval.c_str());
		} else if (optstr.compare("training_algorithm") == 0) {
			this->training_algorithm = atoi(optval.c_str());
		} else if (optstr.compare("sigma_square") == 0) {
			this->sigma_square = atof(optval.c_str());
		} else if (optstr.compare("decoding_algorithm") == 0) {
			this->decoding_algorithm = atoi(optval.c_str());
		} else if (optstr.compare("include_bos_feature") == 0) {
			this->include_bos_feature = atoi(optval.c_str()) == 1 ? true : false;
		} else if (optstr.compare("include_eos_feature") == 0) {
			this->include_eos_feature = atoi(optval.c_str()) == 1 ? true : false;
		} else if (optstr.compare("iterations") == 0) {
			this->iterations = atoi(optval.c_str());
		} else if (optstr.compare("init_params_value") == 0) {
			this->init_params_value = atof(optval.c_str());
		} else if (optstr.compare("randomize_params") == 0) {
			this->randomize_params = atoi(optval.c_str()) == 1 ? true : false;
		} else if (optstr.compare("regularize") == 0) {
			this->regularize = atoi(optval.c_str()) == 1 ? true : false;
		} else if (optstr.compare("is_scaling") == 0) {
			this->is_scaling = atoi(optval.c_str()) == 1 ? true : false;
		} else if (optstr.compare("mhessian") == 0) {
			this->mhessian = atoi(optval.c_str());
		} else if (optstr.compare("epsilon") == 0) {
			this->epsilon = atof(optval.c_str());
		} else if (optstr.compare("optimizer") == 0) {
			this->optimizer = atoi(optval.c_str());
		} 
	}
}

void Options::dump()
{
	std::cout << "Model file      : " << model_file << std::endl;
	
	if (istraining) {
	std::cout << "Topology file   : " << topology_file << std::endl;
	std::cout << "Training file   : " << training_file << std::endl;
	std::cout << "Window size     : " << 2*window_size+1 << std::endl;
	std::cout << "Sigma-square    : " << sigma_square << std::endl;
	std::cout << "Epsilon         : " << epsilon << std::endl;
	std::cout << "Init parameters : " << init_params_value << std::endl;
	std::cout << "Iterations      : " << iterations << std::endl;
	}
	if (istesting) {
	std::string algo;
	if (decoding_algorithm == VITERBI)
		algo = "viterbi";
	else algo = "posterior";
	std::cout << "Testing file    : " << testing_file << std::endl;
	std::cout << "Decoding        : " << algo << std::endl;
	std::cout << "Output file     : " << prediction_file << std::endl;

	}
	if (ispredicting) {
	std::cout << "Unlabelled file : " << unlabelled_file << std::endl;
	std::string algo;
	if (decoding_algorithm == VITERBI)
		algo = "viterbi";
	else algo = "posterior";
	std::cout << "Decoding        : " << algo << std::endl;
	std::cout << "Output file     : " << prediction_file << std::endl;
	}

}

}
