/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * Model.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <iostream>
#include <cstdlib>
#include "Model.h"
#include "Tokenizer.h"

namespace models
{

	Model::Model () {
	}

	Model::Model (options::Options * options) {
		this->options = options;
		options->dump();
		if (options->istraining) {

			std::cout << "Reading topology...";
			this->transducer = new grammar::Transducer(options->topology_file);
			this->transducer->model = this;
			std::cout << "done." << std::endl;
			std::cout << "Number of labels : " << transducer->num_labels << std::endl;
                        std::cout << "Number of states : " << transducer->num_states << std::endl;

			this->data = new data::Data();
			this->data->model = this;
		
			this->featureset = new features::FeatureSet();
			this->featureset->model = this;
		
                        std::cout << "Reading training data...";
                        std::cout << std::flush;
			this->data->read(options->training_file);
			std::cout << "done." << std::endl;
                        std::cout << "Number of training sequences: " << this->data->num_sequences << std::endl;

			std::cout << "Generating and assigning features...";
                        std::cout << std::flush;			
			this->featureset->gen_features();
			this->featureset->assign_features();
			std::cout << "done." << std::endl;
			std::cout << "Total number of features      : " << this->featureset->features.size() << std::endl;
                        std::cout << "Number of transition features : " << this->featureset->trans_features.size() << std::endl;
                        std::cout << "Number of state features      : " << (featureset->features.size() - featureset->trans_features.size()) << std::endl;
		} else if (options->istesting) {
                        
			//std::filebuf fb;
                        //fb.open(options->model_file.c_str(), std::ios::in);
                        //std::istream model_file(&fb);
			
			std::ifstream model_file(options->model_file.c_str());
			if(! model_file.is_open()) {
				std::cout << "Error opening/reading model file" << std::endl;
				exit(EXIT_FAILURE);
			}

                        std::cout << "Reading topology from model...";
                        this->transducer = new grammar::Transducer(&model_file);
                        this->transducer->model = this;
                        std::cout << "done." << std::endl;

                        std::cout << "Number of labels : " << transducer->num_labels << std::endl;
                        std::cout << "Number of states : " << transducer->num_states << std::endl;

                        this->data = new data::Data();
                        this->data->model = this;

                        std::cout << "Reading testing data...";
                        std::cout << std::flush;
                        this->data->read(options->testing_file);
                        std::cout << "done." << std::endl;

                        std::cout << "Number of testing sequences: " << this->data->num_sequences << std::endl;
			char buff[64];
			model_file.getline(buff, BUFF_SIZE_LONG);
			std::string line = buff;
			utils::Tokenizer tok(line, " \t\r\n");
			options->window_size = atoi(tok.token(1).c_str());
                        
			std::cout << "Reading features from model...";
                        this->featureset = new features::FeatureSet(&model_file);
                        this->featureset->model = this;
                        std::cout << "done." << std::endl;

                        std::cout << "Assigning features...";
                        std::cout << std::flush;
                        this->featureset->assign_features();
                        std::cout << "done." << std::endl;
                        std::cout << "Total number of features      : " << this->featureset->features.size() << std::endl;
                        std::cout << "Number of transition features : " << this->featureset->trans_features.size() << std::endl;
                        std::cout << "Number of state features      : " << (featureset->features.size() - featureset->trans_features.size()) << std::endl;			
			model_file.close();
			if (options->out_posterior) {
				std::ofstream tmp(options->posterior_out_file.c_str(), std::ios_base::trunc);
				tmp.close();
			}
		} else if (options->ispredicting) {
                        /*
			std::filebuf fb;
                        fb.open(options->model_file.c_str(), std::ios::in);
                        std::istream model_file(&fb);
			*/
			std::ifstream model_file(options->model_file.c_str());
                        if(! model_file.is_open()) {
				std::cout << "Error opening/reading model file" << std::endl;
				exit(EXIT_FAILURE);
			}

                        std::cout << "Reading topology from model...";
                        this->transducer = new grammar::Transducer(&model_file);
                        this->transducer->model = this;
                        std::cout << "done." << std::endl;

                        std::cout << "Number of labels : " << transducer->num_labels << std::endl;
                        std::cout << "Number of states : " << transducer->num_states << std::endl;

                        this->data = new data::Data();
                        this->data->model = this;

                        std::cout << "Reading unlabelled data...";
                        std::cout << std::flush;
                        this->data->read(options->unlabelled_file);
                        std::cout << "done." << std::endl;

                        std::cout << "Number of unlabelled sequences: " << this->data->num_sequences << std::endl;
  		        char buff[64];
 			model_file.getline(buff, BUFF_SIZE_LONG);
			std::string line = buff;
			utils::Tokenizer tok(line, " \t\r\n");
			options->window_size = atoi(tok.token(1).c_str());
                        
			std::cout << "Reading features from model...";
			std::cout << std::flush;
			this->featureset = new features::FeatureSet(&model_file);
                        this->featureset->model = this;
                        std::cout << "done." << std::endl;
			std::cout << std::flush;


                        std::cout << "Assigning features...";
                        std::cout << std::flush;
                        this->featureset->assign_features();
                        std::cout << "done." << std::endl;
                        std::cout << "Total number of features      : " << this->featureset->features.size() << std::endl;
                        std::cout << "Number of transition features : " << this->featureset->trans_features.size() << std::endl;
                        std::cout << "Number of state features      : " << (featureset->features.size() - featureset->trans_features.size()) << std::endl;
			model_file.close();
			if (options->out_posterior) {
                           std::ofstream tmp(options->posterior_out_file.c_str(), std::ios_base::trunc);
                           tmp.close();
                        }
		}
	}
	
	Model::~Model(){
		
		if (transducer != NULL)
			delete transducer;
		if (data != NULL)
			delete data;
		if (featureset != NULL)
			delete featureset;
		if (trainer != NULL)
			delete trainer;
		if (decoder != NULL)
			delete decoder;
	}
}

