/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * OptimizerLBFGS.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <iostream>
#include "OptimizerLBFGS.h"

namespace training
{

/*--------------------------------------------------------------------------------*/
extern "C" {
    // interface to LBFGS optimization written in FORTRAN
    extern void lbfgs(int * n, int * m, double * x, double * f, double * g,
		       int * diagco, double * diag, int * iprint, double * eps,
		       double * xtol, double * w, int * iflag);		       
}
/*--------------------------------------------------------------------------------*/

OptimizerLBFGS::OptimizerLBFGS(int num_params, int mhessian, double epsilon)
{
	this->num_params = num_params;
	this->mhessian = mhessian;
	this->epsilon = epsilon;
	
	this->iprint = new int[2];
	this->iprint[0] = -1;
    this->iprint[1] = 0;
    
    this->iflag = 0;
	this->diagco = 0;
	this->xtol = 1.0e-16;
	
	diag = new double[num_params];
	int ws_size = num_params * (2 * this->mhessian + 1) + 
		  2 * this->mhessian;
	ws = new double[ws_size];
}

int OptimizerLBFGS::optimize()
{
	
	std::cout << "Starting LBFGS optimization" << std::endl;
	lbfgs(&(this->num_params), &(this->mhessian), parameters, &fval, gradient, &(this->diagco), 
								diag, iprint, &(this->epsilon), &xtol, ws, &iflag);

								
	return iflag;
}

}

