/*
 * Copyright (C) 2009 Castrense Savojardo
 * 	Bologna Biocomputing Group
 * 	University of Bologna, Italy
 *  savojard@biocomp.unibo.it
 * 
 * Tokenizer.cpp - This file is part of BioCRF
 * 
 * BioCRF is a free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * BioCRF is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BioCRF; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 */
#include <string>
#include <vector>
#include "Tokenizer.h"

namespace utils {

Tokenizer::Tokenizer(std::string str, std::string seperators)
{
	parse(str, seperators);
}

void Tokenizer::parse(std::string str, std::string seperators)
{
	int n = str.length();
    int start, stop;
    
    start = str.find_first_not_of(seperators);
    while (start >= 0 && start < n) {
	stop = str.find_first_of(seperators, start);
	if (stop < 0 || stop > n) {
	    stop = n;
	}
	
	tokens.push_back(str.substr(start, stop - start));	
	start = str.find_first_not_of(seperators, stop + 1);
    }
    
    start_scan();
}
    
int Tokenizer::count_tokens()
{
	return tokens.size();
}

std::string Tokenizer::next_token()
{
	if (idx >= 0 && idx < tokens.size()) {
	return tokens[idx++];
    } else {
	return "";
    }
}
   
void Tokenizer::start_scan()
{
	idx = 0;
}

std::string Tokenizer::token(int i)
{
	if (i >= 0 && i < tokens.size()) {
	return tokens[i];
    } else {
	return "";
    }
}

}
